/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.buydetails.web;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.jeeplus.modules.oabuy.entity.OaBuy;
import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.buydetails.entity.BuyDetails;
import com.jeeplus.modules.buydetails.service.BuyDetailsService;

/**
 * 采购明细Controller
 * @author liuw
 * @version 2018-01-16
 */
@Controller
@RequestMapping(value = "${adminPath}/buydetails/buyDetails")
public class BuyDetailsController extends BaseController {

	@Autowired
	private BuyDetailsService buyDetailsService;
	
	@ModelAttribute
	public BuyDetails get(@RequestParam(required=false) String id) {
		BuyDetails entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = buyDetailsService.get(id);
		}
		if (entity == null){
			entity = new BuyDetails();
		}
		return entity;
	}
	
	/**
	 * 采购明细列表页面
	 */
	@RequiresPermissions("buydetails:buyDetails:list")
	@RequestMapping(value = {"list", ""})
	public String list(BuyDetails buyDetails, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<BuyDetails> page = buyDetailsService.findPage(new Page<BuyDetails>(request, response), buyDetails); 
		model.addAttribute("page", page);
		return "modules/buydetails/buyDetailsList";
	}

	/**
	 * 查看，增加，编辑采购明细表单页面
	 */
	/*@RequiresPermissions(value={"buydetails:buyDetails:view","buydetails:buyDetails:add","buydetails:buyDetails:edit"},logical=Logical.OR)*/
	@RequestMapping(value = "form")
	public String form(BuyDetails buyDetails, Model model) {
		model.addAttribute("buyDetails", buyDetails);
		return "modules/buydetails/buyDetailsForm";
	}

	/**
	 * 保存采购明细
	 */
	/*@RequiresPermissions(value={"buydetails:buyDetails:add","buydetails:buyDetails:edit"},logical=Logical.OR)*/
	@RequestMapping(value = "save")
	public String save(BuyDetails buyDetails, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, buyDetails)){
			return form(buyDetails, model);
		}
		if(!buyDetails.getIsNewRecord()){//编辑表单保存
			BuyDetails t = buyDetailsService.get(buyDetails.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(buyDetails, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			buyDetailsService.save(t);//保存
		}else{//新增表单保存
			buyDetailsService.save(buyDetails);//保存
		}
		addMessage(redirectAttributes, "保存采购明细成功");
		return "redirect:"+Global.getAdminPath()+"/buydetails/buyDetails/?repage";
	}
	
	/**
	 * 删除采购明细
	 */
	/*@RequiresPermissions("buydetails:buyDetails:del")*/
	@RequestMapping(value = "delete")
	public String delete(BuyDetails buyDetails, RedirectAttributes redirectAttributes) {
		buyDetailsService.delete(buyDetails);
		addMessage(redirectAttributes, "删除采购明细成功");
		return "redirect:"+Global.getAdminPath()+"/buydetails/buyDetails/?repage";
	}
	
	/**
	 * 批量删除采购明细
	 */
	@RequiresPermissions("buydetails:buyDetails:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			buyDetailsService.delete(buyDetailsService.get(id));
		}
		addMessage(redirectAttributes, "删除采购明细成功");
		return "redirect:"+Global.getAdminPath()+"/buydetails/buyDetails/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("buydetails:buyDetails:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(BuyDetails buyDetails, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "采购明细"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<BuyDetails> page = buyDetailsService.findPage(new Page<BuyDetails>(request, response, -1), buyDetails);
    		new ExportExcel("采购明细", BuyDetails.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出采购明细记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/buydetails/buyDetails/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("buydetails:buyDetails:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<BuyDetails> list = ei.getDataList(BuyDetails.class);
			for (BuyDetails buyDetails : list){
				try{
					buyDetailsService.save(buyDetails);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条采购明细记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条采购明细记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入采购明细失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/buydetails/buyDetails/?repage";
    }
	
	/**
	 * 下载导入采购明细数据模板
	 */
	@RequiresPermissions("buydetails:buyDetails:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "采购明细数据导入模板.xlsx";
    		List<BuyDetails> list = Lists.newArrayList(); 
    		new ExportExcel("采购明细数据", BuyDetails.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/buydetails/buyDetails/?repage";
    }
	
	
	/**
	 * 选择采购申请id
	 */
	@RequestMapping(value = "selectoaBuy")
	public String selectoaBuy(OaBuy oaBuy, String url, String fieldLabels, String fieldKeys, String searchLabel, String searchKey, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<OaBuy> page = buyDetailsService.findPageByoaBuy(new Page<OaBuy>(request, response),  oaBuy);
		try {
			fieldLabels = URLDecoder.decode(fieldLabels, "UTF-8");
			fieldKeys = URLDecoder.decode(fieldKeys, "UTF-8");
			searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
			searchKey = URLDecoder.decode(searchKey, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		model.addAttribute("labelNames", fieldLabels.split("\\|"));
		model.addAttribute("labelValues", fieldKeys.split("\\|"));
		model.addAttribute("fieldLabels", fieldLabels);
		model.addAttribute("fieldKeys", fieldKeys);
		model.addAttribute("url", url);
		model.addAttribute("searchLabel", searchLabel);
		model.addAttribute("searchKey", searchKey);
		model.addAttribute("obj", oaBuy);
		model.addAttribute("page", page);
		return "modules/sys/gridselect";
	}
	

}