/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.project.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.project.entity.Projectgeneral;
import com.jeeplus.modules.project.service.ProjectgeneralService;

/**
 * 项目概况Controller
 * @author 姚儒彬
 * @version 2017-10-11
 */
@Controller
@RequestMapping(value = "${adminPath}/project/projectgeneral")
public class ProjectgeneralController extends BaseController {

	@Autowired
	private ProjectgeneralService projectgeneralService;
	
	@ModelAttribute
	public Projectgeneral get(@RequestParam(required=false) String id) {
		Projectgeneral entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectgeneralService.get(id);
		}
		if (entity == null){
			entity = new Projectgeneral();
		}
		return entity;
	}
	
	/**
	 * 项目概况列表页面
	 */
	@RequiresPermissions("project:projectgeneral:list")
	@RequestMapping(value = {"list", ""})
	public String list(Projectgeneral projectgeneral, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Projectgeneral> page = projectgeneralService.findPage(new Page<Projectgeneral>(request, response), projectgeneral); 
		model.addAttribute("page", page);
		return "modules/project/projectgeneralList";
	}

	/**
	 * 查看，增加，编辑项目概况表单页面
	 */
	@RequiresPermissions(value={"project:projectgeneral:view","project:projectgeneral:add","project:projectgeneral:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Projectgeneral projectgeneral, Model model) {
		model.addAttribute("projectgeneral", projectgeneral);
		return "modules/project/projectgeneralForm";
	}

	/**
	 * 保存项目概况
	 */
	@RequiresPermissions(value={"project:projectgeneral:add","project:projectgeneral:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Projectgeneral projectgeneral, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, projectgeneral)){
			return form(projectgeneral, model);
		}
		if(!projectgeneral.getIsNewRecord()){//编辑表单保存
			Projectgeneral t = projectgeneralService.get(projectgeneral.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(projectgeneral, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			projectgeneralService.save(t);//保存
		}else{//新增表单保存
			projectgeneralService.save(projectgeneral);//保存
		}
		addMessage(redirectAttributes, "保存项目概况成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectgeneral/?repage";
	}
	
	/**
	 * 删除项目概况
	 */
	@RequiresPermissions("project:projectgeneral:del")
	@RequestMapping(value = "delete")
	public String delete(Projectgeneral projectgeneral, RedirectAttributes redirectAttributes) {
		projectgeneralService.delete(projectgeneral);
		addMessage(redirectAttributes, "删除项目概况成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectgeneral/?repage";
	}
	
	/**
	 * 批量删除项目概况
	 */
	@RequiresPermissions("project:projectgeneral:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			projectgeneralService.delete(projectgeneralService.get(id));
		}
		addMessage(redirectAttributes, "删除项目概况成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectgeneral/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("project:projectgeneral:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Projectgeneral projectgeneral, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "项目概况"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Projectgeneral> page = projectgeneralService.findPage(new Page<Projectgeneral>(request, response, -1), projectgeneral);
    		new ExportExcel("项目概况", Projectgeneral.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出项目概况记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectgeneral/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("project:projectgeneral:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Projectgeneral> list = ei.getDataList(Projectgeneral.class);
			for (Projectgeneral projectgeneral : list){
				try{
					projectgeneralService.save(projectgeneral);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条项目概况记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条项目概况记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入项目概况失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectgeneral/?repage";
    }
	
	/**
	 * 下载导入项目概况数据模板
	 */
	@RequiresPermissions("project:projectgeneral:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletRequest request,HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "项目概况数据导入模板.xlsx";
    		List<Projectgeneral> list = Lists.newArrayList();

    		new ExportExcel("项目概况数据", Projectgeneral.class, 1).setDataList(list).write(response, fileName,request.getHeader("USER-AGENT")).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectgeneral/?repage";
    }
	
	
	

}