/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.project.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.project.entity.Workprojectgroup;
import com.jeeplus.modules.project.service.WorkprojectgroupService;

/**
 * 项目组信息表Controller
 * @author yangfan
 * @version 2017-10-11
 */
@Controller
@RequestMapping(value = "${adminPath}/project/workprojectgroup")
public class WorkprojectgroupController extends BaseController {

	@Autowired
	private WorkprojectgroupService workprojectgroupService;
	
	@ModelAttribute
	public Workprojectgroup get(@RequestParam(required=false) String id) {
		Workprojectgroup entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workprojectgroupService.get(id);
		}
		if (entity == null){
			entity = new Workprojectgroup();
		}
		return entity;
	}
	
	/**
	 * 项目组信息表列表页面
	 */
	@RequiresPermissions("project:workprojectgroup:list")
	@RequestMapping(value = {"list", ""})
	public String list(Workprojectgroup workprojectgroup, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Workprojectgroup> page = workprojectgroupService.findPage(new Page<Workprojectgroup>(request, response), workprojectgroup); 
		model.addAttribute("page", page);
		return "modules/project/workprojectgroupList";
	}

	/**
	 * 查看，增加，编辑项目组信息表表单页面
	 */
	@RequiresPermissions(value={"project:workprojectgroup:view","project:workprojectgroup:add","project:workprojectgroup:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Workprojectgroup workprojectgroup, Model model) {
		model.addAttribute("workprojectgroup", workprojectgroup);
		return "modules/project/workprojectgroupForm";
	}

	/**
	 * 保存项目组信息表
	 */
	@RequiresPermissions(value={"project:workprojectgroup:add","project:workprojectgroup:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Workprojectgroup workprojectgroup, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workprojectgroup)){
			return form(workprojectgroup, model);
		}
		if(!workprojectgroup.getIsNewRecord()){//编辑表单保存
			Workprojectgroup t = workprojectgroupService.get(workprojectgroup.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workprojectgroup, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workprojectgroupService.save(t);//保存
		}else{//新增表单保存
			workprojectgroupService.save(workprojectgroup);//保存
		}
		addMessage(redirectAttributes, "保存项目组信息表成功");
		return "redirect:"+Global.getAdminPath()+"/project/workprojectgroup/?repage";
	}
	
	/**
	 * 删除项目组信息表
	 */
	@RequiresPermissions("project:workprojectgroup:del")
	@RequestMapping(value = "delete")
	public String delete(Workprojectgroup workprojectgroup, RedirectAttributes redirectAttributes) {
		workprojectgroupService.delete(workprojectgroup);
		addMessage(redirectAttributes, "删除项目组信息表成功");
		return "redirect:"+Global.getAdminPath()+"/project/workprojectgroup/?repage";
	}
	
	/**
	 * 批量删除项目组信息表
	 */
	@RequiresPermissions("project:workprojectgroup:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workprojectgroupService.delete(workprojectgroupService.get(id));
		}
		addMessage(redirectAttributes, "删除项目组信息表成功");
		return "redirect:"+Global.getAdminPath()+"/project/workprojectgroup/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("project:workprojectgroup:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Workprojectgroup workprojectgroup, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "项目组信息表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Workprojectgroup> page = workprojectgroupService.findPage(new Page<Workprojectgroup>(request, response, -1), workprojectgroup);
    		new ExportExcel("项目组信息表", Workprojectgroup.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出项目组信息表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/workprojectgroup/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("project:workprojectgroup:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Workprojectgroup> list = ei.getDataList(Workprojectgroup.class);
			for (Workprojectgroup workprojectgroup : list){
				try{
					workprojectgroupService.save(workprojectgroup);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条项目组信息表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条项目组信息表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入项目组信息表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/workprojectgroup/?repage";
    }
	
	/**
	 * 下载导入项目组信息表数据模板
	 */
	@RequiresPermissions("project:workprojectgroup:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "项目组信息表数据导入模板.xlsx";
    		List<Workprojectgroup> list = Lists.newArrayList(); 
    		new ExportExcel("项目组信息表数据", Workprojectgroup.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/workprojectgroup/?repage";
    }
	
	
	

}