/**
 * Copyright &copy; 2013-2017 <a href="http://www.rhcncpa.com/">瑞华会计师事务所</a> All rights reserved.
 */
package com.jeeplus.modules.sys.dao;

import java.util.HashMap;
import java.util.List;

import com.jeeplus.modules.sys.entity.Office;
import com.jeeplus.modules.workstaff.entity.WorkStaffBasicInfo;
import org.apache.ibatis.annotations.Param;

import com.jeeplus.common.persistence.CrudDao;
import com.jeeplus.common.persistence.annotation.MyBatisDao;
import com.jeeplus.modules.sys.entity.Role;
import com.jeeplus.modules.sys.entity.User;

/**
 * 用户DAO接口
 * @author jeeplus
 * @version 2014-05-16
 */
@MyBatisDao
public interface UserDao extends CrudDao<User> {
	
	/**
	 * 根据登录名称查询用户
	 * @param user
	 * @return
	 */
	public User getByLoginName(User user);
	public List<User> getByName(User user);

	public User getByUserId(String id);
	public User findUserByNo(@Param("no")String no,@Param("companyId")String companyId);
	public User getByCompanyAndUserId(@Param("id")String id,@Param("companyId")String companyId);
	public List<User>  findUserByRoleId(@Param("roleId")String roleId,@Param("companyId")String companyId);

	/**
	 * 通过OfficeId获取用户列表，仅返回用户id和name（树查询用户时用）
	 * @param user
	 * @return
	 */
	public List<User> findUserByOfficeId(User user);
	/**
	 * 通过Offices获取用户列表，仅返回用户id和name（树查询用户时用）
	 * @param user
	 * @return
	 */
	public List<User> findUserByOffices(User user);
	public List<User> findListByComany(User user);

	/**
	 * 查询全部用户数目
	 * @return
	 */
	public long findAllCount(User user);
	
	/**
	 * 更新用户密码
	 * @param user
	 * @return
	 */
	public int updatePasswordById(User user);

	//更改用户离职之后的状态
	public int updateUserState(User user);
	/**
	 * 更新登录信息，如：登录IP、登录时间
	 * @param user
	 * @return
	 */
	public int updateLoginInfo(User user);

	/**
	 * 删除用户岗位关联数据
	 * @param user
	 * @return
	 */
	public int deleteUserRole(User user);
	
	/**
	 * 插入用户岗位关联数据
	 * @param user
	 * @return
	 */
	public int insertUserRole(User user);
	
	/**
	 * 更新用户信息
	 * @param user
	 * @return
	 */
	public int updateUserInfo(User user);
	public int updateLoginFlag(User user);
	public int updateMo(User user);
	public int updateUserRead(@Param("userId")String userId, @Param("friendId")String friendId);
	public int updateUserReadAll(@Param("userId")String userId, @Param("friendId")String friendId);

	/**
	 * 插入好友
	 */
	public int insertFriend(@Param("id")String id, @Param("userId")String userId, @Param("friendId")String friendId ,
							@Param("status")String status, @Param("type")String type);
	
	/**
	 * 查找好友
	 */
	public User findFriend(@Param("userId")String userId, @Param("friendId")String friendId);
	/**
	 * 查找好友
	 */
	public User findFriendInfo(@Param("userId")String userId, @Param("friendId")String friendId);
	/**
	 * 获取
	 */
	public User findFriendByStatus(@Param("userId")String userId, @Param("friendId")String friendId);
	/**
	 * 删除好友
	 */
	public void deleteFriend(@Param("userId")String userId, @Param("friendId")String friendId);
	/**
	 * 确认好友好友
	 */
	public void friendAudit(@Param("userId")String userId, @Param("friendId")String friendId);
	/**
	 * 拒绝好友
	 */
	public void friendAuditNo(@Param("userId")String userId, @Param("friendId")String friendId);

	/**
	 * 
	 * 获取我的好友列表
	 * 
	 */
	public List<User> findFriends(User currentUser);
	/**
	 *
	 * 获取我的待审批好友列表
	 *
	 */
	public List<User> findFriendsAudit(User currentUser);

	/**
	 * 
	 * 查询用户-->用来添加到常用联系人
	 * 
	 */
	public List<User> searchUsers(User user);
	/**
	 *
	 * 查询用户-->用来添加到常用联系人
	 *
	 */
	public List<User> findFriendByMobileOrName(User user);
	/**
	 *
	 * 查询用户-->用来添加到常用联系人
	 *
	 */
	public List<User> findFriendByMobileOrUserName(User user);
	/**
	 *
	 * 查询用户-->用来添加到常用联系人
	 *
	 */
	public List<User> findUserByMobileOrName(User user);
	/**
	 *
	 * 查询用户-->用来添加到常用联系人
	 *
	 */
	public List<User> findUserByStatus(User user);
	/**
	 *
	 * 查询用户-->用来添加到常用联系人
	 *
	 */
	public List<User> findFriendAll(User user);
	/**
	 *
	 * 查询用户-->用来添加到常用联系人
	 *
	 */
	public List<User> findUserByStatusAudit(User user);

	/**
	 * 
	 */
	
	public List<User>  findListByOffice(User user);
	
	/**
	 * 
	 */
	public List<User> findUserByRole(Role role);

	public User findUserByMobile(User user);


	User getUserById(String createId);

	List queryRole(String officeId);

	//根据岗位获取用户对象
	public List<User> findListByRoleList(User user);

    List<User> queryListByIds(String[] array);

	public List findUserRole(@Param("userId")String userId, @Param("roleId")String roleId);

	public List<User> findUserByRoles(String id);

	public List<User> findUserByUser(String id);

	public Integer userCount(String officeId);

    int countByOffice(User user);

    List<User> findUserByOfficeRole(Office office);

    void updateUserCompanyId(String companyId);

    int insertRole(WorkStaffBasicInfo workStaffBasicInfo);
    int updateRole(WorkStaffBasicInfo workStaffBasicInfo);

	/**
	 * 修改的属性，同步到sys_user
	 * @param user
	 * @return
	 */
	int updateUser(User user);

	String findUserNo(String weChatId);
}
