/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysuseroffice.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.pushinfo.entity.Pushinfo;
import com.jeeplus.modules.pushinfo.service.PushinfoService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sysuseroffice.entity.Useroffice;
import com.jeeplus.modules.sysuseroffice.service.UserofficeService;

/**
 * 用户所在公司Controller
 * @author yangfan
 * @version 2017-08-07
 */
@Controller
@RequestMapping(value = "${adminPath}/sysuseroffice/useroffice")
public class UserofficeController extends BaseController {

	@Autowired
	private UserofficeService userofficeService;
	@Autowired
	private PushinfoService pushinfoService;
	
	@ModelAttribute
	public Useroffice get(@RequestParam(required=false) String id) {
		Useroffice entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = userofficeService.get(id);
		}
		if (entity == null){
			entity = new Useroffice();
		}
		return entity;
	}
	
	/**
	 * 用户所在公司列表页面
	 */
	@RequiresPermissions("sysuseroffice:useroffice:list")
	@RequestMapping(value = {"list", ""})
	public String list(Useroffice useroffice, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Useroffice> page = userofficeService.findPage(new Page<Useroffice>(request, response), useroffice); 
		model.addAttribute("page", page);
		return "modules/sysuseroffice/userofficeList";
	}

	/**
	 * 查看，增加，编辑用户所在公司表单页面
	 */
	@RequiresPermissions(value={"sysuseroffice:useroffice:view","sysuseroffice:useroffice:add","sysuseroffice:useroffice:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Useroffice useroffice, Model model) {
		model.addAttribute("useroffice", useroffice);
		return "modules/sysuseroffice/userofficeForm";
	}

	/**
	 * 保存用户所在公司
	 */
	@RequiresPermissions(value={"sysuseroffice:useroffice:add","sysuseroffice:useroffice:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Useroffice useroffice, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, useroffice)){
			return form(useroffice, model);
		}
		if(!useroffice.getIsNewRecord()){//编辑表单保存
			Useroffice t = userofficeService.get(useroffice.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(useroffice, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			userofficeService.save(t);//保存
		}else{//新增表单保存
			userofficeService.save(useroffice);//保存
		}
		addMessage(redirectAttributes, "保存用户所在公司成功");
		return "redirect:"+Global.getAdminPath()+"/sysuseroffice/useroffice/?repage";
	}
	
	/**
	 * 删除用户所在公司
	 */
	@RequiresPermissions("sysuseroffice:useroffice:del")
	@RequestMapping(value = "delete")
	public String delete(Useroffice useroffice, RedirectAttributes redirectAttributes) {
		userofficeService.delete(useroffice);
		Pushinfo pushinfo = new Pushinfo();
		pushinfo.setPushId(useroffice.getId());
		List<Pushinfo> pushinfos = pushinfoService.findList(pushinfo);
		if (pushinfos!=null && pushinfos.size()!=0){
			pushinfoService.updateDelflagByPushId(useroffice.getId(),"1");
		}
		addMessage(redirectAttributes, "删除用户所在公司成功");
		return "redirect:"+Global.getAdminPath()+"/sysuseroffice/useroffice/?repage";
	}
	
	/**
	 * 批量删除用户所在公司
	 */
	@RequiresPermissions("sysuseroffice:useroffice:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			userofficeService.delete(userofficeService.get(id));
			Pushinfo pushinfo = new Pushinfo();
			pushinfo.setPushId(id);
			List<Pushinfo> pushinfos = pushinfoService.findList(pushinfo);
			if (pushinfos!=null && pushinfos.size()!=0){
				pushinfoService.updateDelflagByPushId(id,"1");
			}
		}
		addMessage(redirectAttributes, "删除用户所在公司成功");
		return "redirect:"+Global.getAdminPath()+"/sysuseroffice/useroffice/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysuseroffice:useroffice:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Useroffice useroffice, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "用户所在公司"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Useroffice> page = userofficeService.findPage(new Page<Useroffice>(request, response, -1), useroffice);
    		new ExportExcel("用户所在公司", Useroffice.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出用户所在公司记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysuseroffice/useroffice/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysuseroffice:useroffice:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Useroffice> list = ei.getDataList(Useroffice.class);
			for (Useroffice useroffice : list){
				try{
					userofficeService.save(useroffice);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条用户所在公司记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条用户所在公司记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入用户所在公司失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysuseroffice/useroffice/?repage";
    }
	
	/**
	 * 下载导入用户所在公司数据模板
	 */
	@RequiresPermissions("sysuseroffice:useroffice:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "用户所在公司数据导入模板.xlsx";
    		List<Useroffice> list = Lists.newArrayList(); 
    		new ExportExcel("用户所在公司数据", Useroffice.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysuseroffice/useroffice/?repage";
    }
	
	
	

}