/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.syswarning.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.syswarning.entity.SysWarningLog;
import com.jeeplus.modules.syswarning.service.SysWarningLogService;

/**
 * 预警日志Controller
 * @author ssrh
 * @version 2018-08-16
 */
@Controller
@RequestMapping(value = "${adminPath}/syswarning/sysWarningLog")
public class SysWarningLogController extends BaseController {

	@Autowired
	private SysWarningLogService sysWarningLogService;
	
	@ModelAttribute
	public SysWarningLog get(@RequestParam(required=false) String id) {
		SysWarningLog entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysWarningLogService.get(id);
		}
		if (entity == null){
			entity = new SysWarningLog();
		}
		return entity;
	}
	
	/**
	 * 预警日志列表页面
	 */

	@RequestMapping(value = {"list", ""})
	public String list(SysWarningLog sysWarningLog, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysWarningLog> page = sysWarningLogService.findPage(new Page<SysWarningLog>(request, response), sysWarningLog); 
		model.addAttribute("page", page);
		return "modules/syswarning/sysWarningLogList";
	}

	/**
	 * 查看，增加，编辑预警日志表单页面
	 */
	@RequiresPermissions(value={"syswarning:sysWarningLog:view","syswarning:sysWarningLog:add","syswarning:sysWarningLog:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysWarningLog sysWarningLog, Model model) {
		model.addAttribute("sysWarningLog", sysWarningLog);
		return "modules/syswarning/sysWarningLogForm";
	}

	/**
	 * 保存预警日志
	 */
	@RequiresPermissions(value={"syswarning:sysWarningLog:add","syswarning:sysWarningLog:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysWarningLog sysWarningLog, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysWarningLog)){
			return form(sysWarningLog, model);
		}
		if(!sysWarningLog.getIsNewRecord()){//编辑表单保存
			SysWarningLog t = sysWarningLogService.get(sysWarningLog.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysWarningLog, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysWarningLogService.save(t);//保存
		}else{//新增表单保存
			sysWarningLogService.save(sysWarningLog);//保存
		}
		addMessage(redirectAttributes, "保存预警日志成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningLog/?repage";
	}
	
	/**
	 * 删除预警日志
	 */
	@RequiresPermissions("syswarning:sysWarningLog:del")
	@RequestMapping(value = "delete")
	public String delete(SysWarningLog sysWarningLog, RedirectAttributes redirectAttributes) {
		sysWarningLogService.delete(sysWarningLog);
		addMessage(redirectAttributes, "删除预警日志成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningLog/?repage";
	}
	
	/**
	 * 批量删除预警日志
	 */
	@RequiresPermissions("syswarning:sysWarningLog:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysWarningLogService.delete(sysWarningLogService.get(id));
		}
		addMessage(redirectAttributes, "删除预警日志成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningLog/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("syswarning:sysWarningLog:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysWarningLog sysWarningLog, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "预警日志"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysWarningLog> page = sysWarningLogService.findPage(new Page<SysWarningLog>(request, response, -1), sysWarningLog);
    		new ExportExcel("预警日志", SysWarningLog.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出预警日志记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningLog/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("syswarning:sysWarningLog:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysWarningLog> list = ei.getDataList(SysWarningLog.class);
			for (SysWarningLog sysWarningLog : list){
				try{
					sysWarningLogService.save(sysWarningLog);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条预警日志记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条预警日志记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入预警日志失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningLog/?repage";
    }
	
	/**
	 * 下载导入预警日志数据模板
	 */
	@RequiresPermissions("syswarning:sysWarningLog:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "预警日志数据导入模板.xlsx";
    		List<SysWarningLog> list = Lists.newArrayList(); 
    		new ExportExcel("预警日志数据", SysWarningLog.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningLog/?repage";
    }
	
	
	

}