/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.syswarning.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.syswarning.entity.SysWarningRole;
import com.jeeplus.modules.syswarning.service.SysWarningRoleService;

/**
 * 系统预警Controller
 * @author chenyuping
 * @version 2018-08-14
 */
@Controller
@RequestMapping(value = "${adminPath}/syswarning/sysWarningRole")
public class SysWarningRoleController extends BaseController {

	@Autowired
	private SysWarningRoleService sysWarningRoleService;
	
	@ModelAttribute
	public SysWarningRole get(@RequestParam(required=false) String id) {
		SysWarningRole entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysWarningRoleService.get(id);
		}
		if (entity == null){
			entity = new SysWarningRole();
		}
		return entity;
	}
	
	/**
	 * 系统预警列表页面
	 */
	@RequiresPermissions("syswarning:sysWarningRole:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysWarningRole sysWarningRole, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysWarningRole> page = sysWarningRoleService.findPage(new Page<SysWarningRole>(request, response), sysWarningRole); 
		model.addAttribute("page", page);
		return "modules/syswarning/sysWarningRoleList";
	}

	/**
	 * 查看，增加，编辑系统预警表单页面
	 */
	@RequiresPermissions(value={"syswarning:sysWarningRole:view","syswarning:sysWarningRole:add","syswarning:sysWarningRole:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysWarningRole sysWarningRole, Model model) {
		model.addAttribute("sysWarningRole", sysWarningRole);
		return "modules/syswarning/sysWarningRoleForm";
	}

	/**
	 * 保存系统预警
	 */
	@RequiresPermissions(value={"syswarning:sysWarningRole:add","syswarning:sysWarningRole:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysWarningRole sysWarningRole, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysWarningRole)){
			return form(sysWarningRole, model);
		}
		if(!sysWarningRole.getIsNewRecord()){//编辑表单保存
			SysWarningRole t = sysWarningRoleService.get(sysWarningRole.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysWarningRole, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysWarningRoleService.save(t);//保存
		}else{//新增表单保存
			sysWarningRoleService.save(sysWarningRole);//保存
		}
		addMessage(redirectAttributes, "保存系统预警成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningRole/?repage";
	}
	
	/**
	 * 删除系统预警
	 */
	@RequiresPermissions("syswarning:sysWarningRole:del")
	@RequestMapping(value = "delete")
	public String delete(SysWarningRole sysWarningRole, RedirectAttributes redirectAttributes) {
		sysWarningRoleService.delete(sysWarningRole);
		addMessage(redirectAttributes, "删除系统预警成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningRole/?repage";
	}
	
	/**
	 * 批量删除系统预警
	 */
	@RequiresPermissions("syswarning:sysWarningRole:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysWarningRoleService.delete(sysWarningRoleService.get(id));
		}
		addMessage(redirectAttributes, "删除系统预警成功");
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningRole/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("syswarning:sysWarningRole:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysWarningRole sysWarningRole, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统预警"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysWarningRole> page = sysWarningRoleService.findPage(new Page<SysWarningRole>(request, response, -1), sysWarningRole);
    		new ExportExcel("系统预警", SysWarningRole.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出系统预警记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningRole/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("syswarning:sysWarningRole:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysWarningRole> list = ei.getDataList(SysWarningRole.class);
			for (SysWarningRole sysWarningRole : list){
				try{
					sysWarningRoleService.save(sysWarningRole);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条系统预警记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条系统预警记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入系统预警失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningRole/?repage";
    }
	
	/**
	 * 下载导入系统预警数据模板
	 */
	@RequiresPermissions("syswarning:sysWarningRole:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "系统预警数据导入模板.xlsx";
    		List<SysWarningRole> list = Lists.newArrayList(); 
    		new ExportExcel("系统预警数据", SysWarningRole.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/syswarning/sysWarningRole/?repage";
    }
	
	
	

}