/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workappshow.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workappshow.entity.Workappshow;
import com.jeeplus.modules.workappshow.service.WorkappshowService;

/**
 * app展示Controller
 * @author yangfan
 * @version 2017-08-08
 */
@Controller
@RequestMapping(value = "${adminPath}/workappshow/workappshow")
public class WorkappshowController extends BaseController {

	@Autowired
	private WorkappshowService workappshowService;
	
	@ModelAttribute
	public Workappshow get(@RequestParam(required=false) String id) {
		Workappshow entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workappshowService.get(id);
		}
		if (entity == null){
			entity = new Workappshow();
		}
		return entity;
	}
	
	/**
	 * app展示列表页面
	 */
	@RequiresPermissions("workappshow:workappshow:list")
	@RequestMapping(value = {"list", ""})
	public String list(Workappshow workappshow, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Workappshow> page = workappshowService.findPage(new Page<Workappshow>(request, response), workappshow); 
		model.addAttribute("page", page);
		return "modules/workappshow/workappshowList";
	}

	/**
	 * 查看，增加，编辑app展示表单页面
	 */
	@RequiresPermissions(value={"workappshow:workappshow:view","workappshow:workappshow:add","workappshow:workappshow:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Workappshow workappshow, Model model) {
		model.addAttribute("workappshow", workappshow);
		return "modules/workappshow/workappshowForm";
	}

	/**
	 * 保存app展示
	 */
	@RequiresPermissions(value={"workappshow:workappshow:add","workappshow:workappshow:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Workappshow workappshow, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workappshow)){
			return form(workappshow, model);
		}
		if(!workappshow.getIsNewRecord()){//编辑表单保存
			Workappshow t = workappshowService.get(workappshow.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workappshow, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workappshowService.save(t);//保存
		}else{//新增表单保存
			workappshowService.save(workappshow);//保存
		}
		addMessage(redirectAttributes, "保存app展示成功");
		return "redirect:"+Global.getAdminPath()+"/workappshow/workappshow/?repage";
	}
	
	/**
	 * 删除app展示
	 */
	@RequiresPermissions("workappshow:workappshow:del")
	@RequestMapping(value = "delete")
	public String delete(Workappshow workappshow, RedirectAttributes redirectAttributes) {
		workappshowService.delete(workappshow);
		addMessage(redirectAttributes, "删除app展示成功");
		return "redirect:"+Global.getAdminPath()+"/workappshow/workappshow/?repage";
	}
	
	/**
	 * 批量删除app展示
	 */
	@RequiresPermissions("workappshow:workappshow:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workappshowService.delete(workappshowService.get(id));
		}
		addMessage(redirectAttributes, "删除app展示成功");
		return "redirect:"+Global.getAdminPath()+"/workappshow/workappshow/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workappshow:workappshow:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Workappshow workappshow, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "app展示"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Workappshow> page = workappshowService.findPage(new Page<Workappshow>(request, response, -1), workappshow);
    		new ExportExcel("app展示", Workappshow.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出app展示记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workappshow/workappshow/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workappshow:workappshow:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Workappshow> list = ei.getDataList(Workappshow.class);
			for (Workappshow workappshow : list){
				try{
					workappshowService.save(workappshow);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条app展示记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条app展示记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入app展示失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workappshow/workappshow/?repage";
    }
	
	/**
	 * 下载导入app展示数据模板
	 */
	@RequiresPermissions("workappshow:workappshow:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "app展示数据导入模板.xlsx";
    		List<Workappshow> list = Lists.newArrayList(); 
    		new ExportExcel("app展示数据", Workappshow.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workappshow/workappshow/?repage";
    }
	
	
	

}