/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcatalogueitemstock.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcatalogueitemstock.entity.WorkCatalogueItemStock;
import com.jeeplus.modules.workcatalogueitemstock.service.WorkCatalogueItemStockService;

/**
 * 采购申请物品Controller
 * @author lw
 * @version 2018-07-18
 */
@Controller
@RequestMapping(value = "${adminPath}/workcatalogueitemstock/workCatalogueItemStock")
public class WorkCatalogueItemStockController extends BaseController {

	@Autowired
	private WorkCatalogueItemStockService workCatalogueItemStockService;
	
	@ModelAttribute
	public WorkCatalogueItemStock get(@RequestParam(required=false) String id) {
		WorkCatalogueItemStock entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workCatalogueItemStockService.get(id);
		}
		if (entity == null){
			entity = new WorkCatalogueItemStock();
		}
		return entity;
	}
	
	/**
	 * 采购申请物品列表页面
	 */
	@RequiresPermissions("workcatalogueitemstock:workCatalogueItemStock:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkCatalogueItemStock workCatalogueItemStock, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkCatalogueItemStock> page = workCatalogueItemStockService.findPage(new Page<WorkCatalogueItemStock>(request, response), workCatalogueItemStock); 
		model.addAttribute("page", page);
		return "modules/workcatalogueitemstock/workCatalogueItemStockList";
	}

	/**
	 * 查看，增加，编辑采购申请物品表单页面
	 */
	@RequiresPermissions(value={"workcatalogueitemstock:workCatalogueItemStock:view","workcatalogueitemstock:workCatalogueItemStock:add","workcatalogueitemstock:workCatalogueItemStock:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkCatalogueItemStock workCatalogueItemStock, Model model) {
		model.addAttribute("workCatalogueItemStock", workCatalogueItemStock);
		return "modules/workcatalogueitemstock/workCatalogueItemStockForm";
	}

	/**
	 * 保存采购申请物品
	 */
	@RequiresPermissions(value={"workcatalogueitemstock:workCatalogueItemStock:add","workcatalogueitemstock:workCatalogueItemStock:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkCatalogueItemStock workCatalogueItemStock, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workCatalogueItemStock)){
			return form(workCatalogueItemStock, model);
		}
		if(!workCatalogueItemStock.getIsNewRecord()){//编辑表单保存
			WorkCatalogueItemStock t = workCatalogueItemStockService.get(workCatalogueItemStock.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workCatalogueItemStock, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workCatalogueItemStockService.save(t);//保存
		}else{//新增表单保存
			workCatalogueItemStockService.save(workCatalogueItemStock);//保存
		}
		addMessage(redirectAttributes, "保存采购申请物品成功");
		return "redirect:"+Global.getAdminPath()+"/workcatalogueitemstock/workCatalogueItemStock/?repage";
	}
	
	/**
	 * 删除采购申请物品
	 */
	@RequiresPermissions("workcatalogueitemstock:workCatalogueItemStock:del")
	@RequestMapping(value = "delete")
	public String delete(WorkCatalogueItemStock workCatalogueItemStock, RedirectAttributes redirectAttributes) {
		workCatalogueItemStockService.delete(workCatalogueItemStock);
		addMessage(redirectAttributes, "删除采购申请物品成功");
		return "redirect:"+Global.getAdminPath()+"/workcatalogueitemstock/workCatalogueItemStock/?repage";
	}
	
	/**
	 * 批量删除采购申请物品
	 */
	@RequiresPermissions("workcatalogueitemstock:workCatalogueItemStock:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workCatalogueItemStockService.delete(workCatalogueItemStockService.get(id));
		}
		addMessage(redirectAttributes, "删除采购申请物品成功");
		return "redirect:"+Global.getAdminPath()+"/workcatalogueitemstock/workCatalogueItemStock/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcatalogueitemstock:workCatalogueItemStock:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkCatalogueItemStock workCatalogueItemStock, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "采购申请物品"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkCatalogueItemStock> page = workCatalogueItemStockService.findPage(new Page<WorkCatalogueItemStock>(request, response, -1), workCatalogueItemStock);
    		new ExportExcel("采购申请物品", WorkCatalogueItemStock.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出采购申请物品记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcatalogueitemstock/workCatalogueItemStock/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcatalogueitemstock:workCatalogueItemStock:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkCatalogueItemStock> list = ei.getDataList(WorkCatalogueItemStock.class);
			for (WorkCatalogueItemStock workCatalogueItemStock : list){
				try{
					workCatalogueItemStockService.save(workCatalogueItemStock);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条采购申请物品记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条采购申请物品记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入采购申请物品失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcatalogueitemstock/workCatalogueItemStock/?repage";
    }
	
	/**
	 * 下载导入采购申请物品数据模板
	 */
	@RequiresPermissions("workcatalogueitemstock:workCatalogueItemStock:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "采购申请物品数据导入模板.xlsx";
    		List<WorkCatalogueItemStock> list = Lists.newArrayList(); 
    		new ExportExcel("采购申请物品数据", WorkCatalogueItemStock.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcatalogueitemstock/workCatalogueItemStock/?repage";
    }
	
	
	

}