/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcommongroup.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.common.json.AjaxJson;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcommongroup.entity.WorkCommonGroup;
import com.jeeplus.modules.workcommongroup.service.WorkCommonGroupService;

/**
 * 常用组管理Controller
 * @author ssrh
 * @version 2018-07-16
 */
@Controller
@RequestMapping(value = "${adminPath}/workcommongroup/workCommonGroup")
public class WorkCommonGroupController extends BaseController {

	@Autowired
	private WorkCommonGroupService workCommonGroupService;
	
	@ModelAttribute
	public WorkCommonGroup get(@RequestParam(required=false) String id) {
		WorkCommonGroup entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workCommonGroupService.get(id);
		}
		if (entity == null){
			entity = new WorkCommonGroup();
		}
		return entity;
	}
	
	/**
	 * 常用组列表页面
	 */
	@RequiresPermissions("workcommongroup:workCommonGroup:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkCommonGroup workCommonGroup, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkCommonGroup> page = workCommonGroupService.findPage(new Page<WorkCommonGroup>(request, response), workCommonGroup); 
		model.addAttribute("page", page);
		return "modules/workcommongroup/workCommonGroupList";
	}

	/**
	 * 查看，增加，编辑常用组表单页面
	 */
	@RequiresPermissions(value={"workcommongroup:workCommonGroup:view","workcommongroup:workCommonGroup:add","workcommongroup:workCommonGroup:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkCommonGroup workCommonGroup, Model model) {
	    if(workCommonGroup!=null&&StringUtils.isNotBlank(workCommonGroup.getId())){
            workCommonGroupService.queryDetails(workCommonGroup);
        }
		model.addAttribute("workCommonGroup", workCommonGroup);
		return "modules/workcommongroup/workCommonGroupForm";
	}
	/**
	 * 查看，增加，编辑常用组表单页面
	 */
	@RequiresPermissions(value={"workcommongroup:workCommonGroup:view"},logical=Logical.OR)
	@RequestMapping(value = "view")
	public String view(WorkCommonGroup workCommonGroup, Model model) {
	    if(workCommonGroup!=null&&StringUtils.isNotBlank(workCommonGroup.getId())){
            workCommonGroupService.queryDetails(workCommonGroup);
        }
		model.addAttribute("workCommonGroup", workCommonGroup);
		return "modules/workcommongroup/workCommonGroupView";
	}

	/**
	 * 保存常用组
	 */
	@RequiresPermissions(value={"workcommongroup:workCommonGroup:add","workcommongroup:workCommonGroup:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkCommonGroup workCommonGroup, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workCommonGroup)){
			return form(workCommonGroup, model);
		}
		if(!workCommonGroup.getIsNewRecord()){//编辑表单保存
			WorkCommonGroup t = workCommonGroupService.get(workCommonGroup.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workCommonGroup, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workCommonGroupService.save(t);//保存
		}else{//新增表单保存
			workCommonGroupService.save(workCommonGroup);//保存
		}
		addMessage(redirectAttributes, "保存常用组成功");
		return "redirect:"+Global.getAdminPath()+"/workcommongroup/workCommonGroup/?repage";
	}
	
	/**
	 * 删除常用组
	 */
	@RequiresPermissions("workcommongroup:workCommonGroup:del")
	@RequestMapping(value = "delete")
	public String delete(WorkCommonGroup workCommonGroup, RedirectAttributes redirectAttributes) {
		workCommonGroupService.delete(workCommonGroup);
		addMessage(redirectAttributes, "删除常用组成功");
		return "redirect:"+Global.getAdminPath()+"/workcommongroup/workCommonGroup/?repage";
	}
	
	/**
	 * 批量删除常用组
	 */
	@RequiresPermissions("workcommongroup:workCommonGroup:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workCommonGroupService.delete(workCommonGroupService.get(id));
		}
		addMessage(redirectAttributes, "删除常用组成功");
		return "redirect:"+Global.getAdminPath()+"/workcommongroup/workCommonGroup/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcommongroup:workCommonGroup:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkCommonGroup workCommonGroup, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "常用组"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkCommonGroup> page = workCommonGroupService.findPage(new Page<WorkCommonGroup>(request, response, -1), workCommonGroup);
    		new ExportExcel("常用组", WorkCommonGroup.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出常用组记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommongroup/workCommonGroup/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcommongroup:workCommonGroup:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkCommonGroup> list = ei.getDataList(WorkCommonGroup.class);
			for (WorkCommonGroup workCommonGroup : list){
				try{
					workCommonGroupService.save(workCommonGroup);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条常用组记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条常用组记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入常用组失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommongroup/workCommonGroup/?repage";
    }
	
	/**
	 * 下载导入常用组数据模板
	 */
	@RequiresPermissions("workcommongroup:workCommonGroup:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "常用组数据导入模板.xlsx";
    		List<WorkCommonGroup> list = Lists.newArrayList(); 
    		new ExportExcel("常用组数据", WorkCommonGroup.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommongroup/workCommonGroup/?repage";
    }

    @RequestMapping("selectList")
    public String gridSelect(WorkCommonGroup workCommonGroup,Model model,String searchLabel,HttpServletRequest request,HttpServletResponse response){
        Page<WorkCommonGroup> page = workCommonGroupService.findPage(new Page<WorkCommonGroup>(request,response), workCommonGroup);
        model.addAttribute("obj", workCommonGroup);
        model.addAttribute("page", page);
        model.addAttribute("searchLabel",searchLabel);
        return "modules/sys/gridselectCommonGroup";
    }

    @RequestMapping("queryDetail")
    @ResponseBody
    public AjaxJson queryDetail(String id){
        AjaxJson ajaxJson = new AjaxJson();
        WorkCommonGroup workCommonGroup = new WorkCommonGroup(id);
        workCommonGroupService.queryDetails(workCommonGroup);
        ajaxJson.getBody().put("userList",workCommonGroup.getUserList());
        return ajaxJson;
    }
}