/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcommunicaterecord.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workcommunicaterecord.entity.WorkCommunicateContent;
import com.jeeplus.modules.workcommunicaterecord.service.WorkCommunicateContentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 沟通内容Controller
 * @author 马鹏波
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcommunicaterecord/workCommunicateContent")
public class WorkCommunicateContentController extends BaseController {

	@Autowired
	private WorkCommunicateContentService workCommunicateContentService;
	
	@ModelAttribute
	public WorkCommunicateContent get(@RequestParam(required=false) String id) {
		WorkCommunicateContent entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workCommunicateContentService.get(id);
		}
		if (entity == null){
			entity = new WorkCommunicateContent();
		}
		return entity;
	}
	
	/**
	 * 沟通内容列表页面
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateContent:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkCommunicateContent workCommunicateContent, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkCommunicateContent> page = workCommunicateContentService.findPage(new Page<WorkCommunicateContent>(request, response), workCommunicateContent); 
		model.addAttribute("page", page);
		return "modules/workcommunicaterecord/workCommunicateContentList";
	}

	/**
	 * 查看，增加，编辑沟通内容表单页面
	 */
//	@RequiresPermissions(value={"workcommunicaterecord:workCommunicateContent:view","workcommunicaterecord:workCommunicateContent:add","workcommunicaterecord:workCommunicateContent:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkCommunicateContent workCommunicateContent, Model model) {
		model.addAttribute("workCommunicateContent", workCommunicateContent);
		return "modules/workcommunicaterecord/workCommunicateContentForm";
	}

	/**
	 * 保存沟通内容
	 */
//	@RequiresPermissions(value={"workcommunicaterecord:workCommunicateContent:add","workcommunicaterecord:workCommunicateContent:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkCommunicateContent workCommunicateContent, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workCommunicateContent)){
			return form(workCommunicateContent, model);
		}
		if(!workCommunicateContent.getIsNewRecord()){//编辑表单保存
			WorkCommunicateContent t = workCommunicateContentService.get(workCommunicateContent.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workCommunicateContent, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workCommunicateContentService.save(t);//保存
		}else{//新增表单保存
			workCommunicateContentService.save(workCommunicateContent);//保存
		}
		addMessage(redirectAttributes, "保存沟通内容成功");
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateContent/?repage";
	}
	
	/**
	 * 删除沟通内容
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateContent:del")
	@RequestMapping(value = "delete")
	public String delete(WorkCommunicateContent workCommunicateContent, RedirectAttributes redirectAttributes) {
		workCommunicateContentService.delete(workCommunicateContent);
		addMessage(redirectAttributes, "删除沟通内容成功");
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateContent/?repage";
	}
	
	/**
	 * 批量删除沟通内容
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateContent:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workCommunicateContentService.delete(workCommunicateContentService.get(id));
		}
		addMessage(redirectAttributes, "删除沟通内容成功");
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateContent/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateContent:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkCommunicateContent workCommunicateContent, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "沟通内容"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkCommunicateContent> page = workCommunicateContentService.findPage(new Page<WorkCommunicateContent>(request, response, -1), workCommunicateContent);
    		new ExportExcel("沟通内容", WorkCommunicateContent.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出沟通内容记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateContent/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateContent:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkCommunicateContent> list = ei.getDataList(WorkCommunicateContent.class);
			for (WorkCommunicateContent workCommunicateContent : list){
				try{
					workCommunicateContentService.save(workCommunicateContent);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条沟通内容记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条沟通内容记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入沟通内容失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateContent/?repage";
    }
	
	/**
	 * 下载导入沟通内容数据模板
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateContent:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "沟通内容数据导入模板.xlsx";
    		List<WorkCommunicateContent> list = Lists.newArrayList(); 
    		new ExportExcel("沟通内容数据", WorkCommunicateContent.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateContent/?repage";
    }
	
	
	

}