/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentContractinfo;
import com.jeeplus.modules.workcontent.service.WorkContentContractinfoService;

/**
 * 合同登记Controller
 * @author ssrh
 * @version 2018-06-12
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentContractinfo")
public class WorkContentContractinfoController extends BaseController {

	@Autowired
	private WorkContentContractinfoService workContentContractinfoService;
	
	@ModelAttribute
	public WorkContentContractinfo get(@RequestParam(required=false) String id) {
		WorkContentContractinfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentContractinfoService.get(id);
		}
		if (entity == null){
			entity = new WorkContentContractinfo();
		}
		return entity;
	}
	
	/**
	 * 合同登记列表页面
	 */
	@RequiresPermissions("workcontent:workContentContractinfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentContractinfo workContentContractinfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentContractinfo> page = workContentContractinfoService.findPage(new Page<WorkContentContractinfo>(request, response), workContentContractinfo); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentContractinfoList";
	}

	/**
	 * 查看，增加，编辑合同登记表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentContractinfo workContentContractinfo,String view, Model model) {
        WorkContentContractinfo oldRecord = null;
        if(StringUtils.isNotBlank(workContentContractinfo.getContentId())){
            List<WorkContentContractinfo> list = workContentContractinfoService.findList(workContentContractinfo);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
                //查询条款
                workContentContractinfoService.queryTerms(oldRecord);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentContractinfo", oldRecord);
        }else {
            model.addAttribute("workContentContractinfo", workContentContractinfo);
        }
        String viewPath = "modules/workcontent/workContentContractinfoForm";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存合同登记
	 */
	@RequiresPermissions(value={"workcontent:workContentContractinfo:add","workcontent:workContentContractinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContentContractinfo workContentContractinfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentContractinfo)){
			return form(workContentContractinfo,null, model);
		}
		if(!workContentContractinfo.getIsNewRecord()){//编辑表单保存
			WorkContentContractinfo t = workContentContractinfoService.get(workContentContractinfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentContractinfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentContractinfoService.save(t);//保存
		}else{//新增表单保存
			workContentContractinfoService.save(workContentContractinfo);//保存
		}
		addMessage(redirectAttributes, "保存合同登记成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContractinfo/?repage";
	}
	
	/**
	 * 删除合同登记
	 */
	@RequiresPermissions("workcontent:workContentContractinfo:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContentContractinfo workContentContractinfo, RedirectAttributes redirectAttributes) {
		workContentContractinfoService.delete(workContentContractinfo);
		addMessage(redirectAttributes, "删除合同登记成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContractinfo/?repage";
	}
	
	/**
	 * 批量删除合同登记
	 */
	@RequiresPermissions("workcontent:workContentContractinfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentContractinfoService.delete(workContentContractinfoService.get(id));
		}
		addMessage(redirectAttributes, "删除合同登记成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContractinfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentContractinfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentContractinfo workContentContractinfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合同登记"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentContractinfo> page = workContentContractinfoService.findPage(new Page<WorkContentContractinfo>(request, response, -1), workContentContractinfo);
    		new ExportExcel("合同登记", WorkContentContractinfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出合同登记记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContractinfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentContractinfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentContractinfo> list = ei.getDataList(WorkContentContractinfo.class);
			for (WorkContentContractinfo workContentContractinfo : list){
				try{
					workContentContractinfoService.save(workContentContractinfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条合同登记记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条合同登记记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入合同登记失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContractinfo/?repage";
    }
	
	/**
	 * 下载导入合同登记数据模板
	 */
	@RequiresPermissions("workcontent:workContentContractinfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合同登记数据导入模板.xlsx";
    		List<WorkContentContractinfo> list = Lists.newArrayList(); 
    		new ExportExcel("合同登记数据", WorkContentContractinfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContractinfo/?repage";
    }
	
	
	

}