/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workeffectdetail.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workeffectdetail.entity.WorkEffectDetail;
import com.jeeplus.modules.workeffectdetail.service.WorkEffectDetailService;

/**
 * 绩效明细Controller
 * @author ssrh
 * @version 2018-09-11
 */
@Controller
@RequestMapping(value = "${adminPath}/workeffectdetail/workEffectDetail")
public class WorkEffectDetailController extends BaseController {

	@Autowired
	private WorkEffectDetailService workEffectDetailService;
	@Autowired
	private HttpServletRequest request;
	@ModelAttribute
	public WorkEffectDetail get(@RequestParam(required=false) String id) {
		WorkEffectDetail entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workEffectDetailService.get(id);
		}
		if (entity == null){
			entity = new WorkEffectDetail();
		}
		return entity;
	}
	
	/**
	 * 绩效明细列表页面
	 */
	@RequiresPermissions("workeffectdetail:workEffectDetail:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkEffectDetail workEffectDetail, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkEffectDetail> page = workEffectDetailService.findPage(new Page<WorkEffectDetail>(request, response), workEffectDetail); 
		model.addAttribute("page", page);
		return "modules/workeffectdetail/workEffectDetailList";
	}

	/**
	 * 查看，增加，编辑绩效明细表单页面
	 */
	@RequiresPermissions(value={"workeffectdetail:workEffectDetail:view","workeffectdetail:workEffectDetail:add","workeffectdetail:workEffectDetail:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkEffectDetail workEffectDetail, Model model) {
		model.addAttribute("workEffectDetail", workEffectDetail);
		return "modules/workeffectdetail/workEffectDetailForm";
	}

	/**
	 * 保存绩效明细
	 */
	@RequiresPermissions(value={"workeffectdetail:workEffectDetail:add","workeffectdetail:workEffectDetail:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkEffectDetail workEffectDetail, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workEffectDetail)){
			return form(workEffectDetail, model);
		}
		if(!workEffectDetail.getIsNewRecord()){//编辑表单保存
			WorkEffectDetail t = workEffectDetailService.get(workEffectDetail.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workEffectDetail, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workEffectDetailService.save(t);//保存
		}else{//新增表单保存
			workEffectDetailService.save(workEffectDetail);//保存
		}
		addMessage(redirectAttributes, "保存绩效明细成功");
		return "redirect:"+Global.getAdminPath()+"/workeffectdetail/workEffectDetail/?repage";
	}
	
	/**
	 * 删除绩效明细
	 */
	@RequiresPermissions("workeffectdetail:workEffectDetail:del")
	@RequestMapping(value = "delete")
	public String delete(WorkEffectDetail workEffectDetail, RedirectAttributes redirectAttributes) {
		workEffectDetailService.delete(workEffectDetail);
		addMessage(redirectAttributes, "删除绩效明细成功");
		return "redirect:"+Global.getAdminPath()+"/workeffectdetail/workEffectDetail/?repage";
	}
	
	/**
	 * 批量删除绩效明细
	 */
	@RequiresPermissions("workeffectdetail:workEffectDetail:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workEffectDetailService.delete(workEffectDetailService.get(id));
		}
		addMessage(redirectAttributes, "删除绩效明细成功");
		return "redirect:"+Global.getAdminPath()+"/workeffectdetail/workEffectDetail/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workeffectdetail:workEffectDetail:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkEffectDetail workEffectDetail, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "绩效明细"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkEffectDetail> page = workEffectDetailService.findPage(new Page<WorkEffectDetail>(request, response, -1), workEffectDetail);
    		new ExportExcel("绩效明细", WorkEffectDetail.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出绩效明细记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workeffectdetail/workEffectDetail/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	//@RequiresPermissions("workeffectdetail:workEffectDetail:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkEffectDetail> list = ei.getDataList(WorkEffectDetail.class);
			for (WorkEffectDetail workEffectDetail : list){
				try{
					workEffectDetailService.save(workEffectDetail);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条绩效明细记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条绩效明细记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入绩效明细失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workeffectdetail/workEffectDetail/?repage";
    }
	
	/**
	 * 下载导入绩效明细数据模板
	 */
	//@RequiresPermissions("workeffectdetail:workEffectDetail:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "绩效明细数据导入模板.xlsx";
    		List<WorkEffectDetail> list = Lists.newArrayList(); 
    		new ExportExcel("绩效明细数据", WorkEffectDetail.class, 1).setDataList(list).write(response, fileName,request.getHeader("USER-AGENT")).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workeffectdetail/workEffectDetail/?repage";
    }
	
	
	

}