/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workexplore.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workexplore.entity.WorkExplore;
import com.jeeplus.modules.workexplore.service.WorkExploreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 现场踏勘记录Controller
 * @author 马鹏波
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workexplore/workExplore")
public class WorkExploreController extends BaseController {

	@Autowired
	private WorkExploreService workExploreService;
	
	@ModelAttribute
	public WorkExplore get(@RequestParam(required=false) String id) {
		WorkExplore entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workExploreService.get(id);
		}
		if (entity == null){
			entity = new WorkExplore();
		}
		return entity;
	}
	
	/**
	 * 现场踏勘记录列表页面
	 */
//	@RequiresPermissions("workexplore:workExplore:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkExplore workExplore, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkExplore> page = workExploreService.findPage(new Page<WorkExplore>(request, response), workExplore); 
		model.addAttribute("page", page);
		return "modules/workexplore/workExploreList";
	}

	/**
	 * 查看，增加，编辑现场踏勘记录表单页面
	 */
//	@RequiresPermissions(value={"workexplore:workExplore:view","workexplore:workExplore:add","workexplore:workExplore:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkExplore workExplore,HttpServletRequest request, Model model) {
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/workexplore/workExploreFormView";
		}else{
			view = "modules/workexplore/workExploreForm";
		}
		if(!Strings.isNullOrEmpty(workExplore.getProjectId())){
			workExplore = workExploreService.findListByProId(workExplore);
		}
		model.addAttribute("workExplore", workExplore);
		return view;
	}

	/**
	 * 保存现场踏勘记录
	 */
//	@RequiresPermissions(value={"workexplore:workExplore:add","workexplore:workExplore:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkExplore workExplore, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workExplore)){
			return form(workExplore,request, model);
		}
		if(!workExplore.getIsNewRecord()){//编辑表单保存
			WorkExplore t = workExploreService.get(workExplore.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workExplore, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workExploreService.save(t);//保存
		}else{//新增表单保存
			workExploreService.save(workExplore);//保存
		}
		addMessage(redirectAttributes, "保存现场踏勘记录成功");
		return "redirect:"+Global.getAdminPath()+"/workexplore/workExplore/?repage";
	}
	
	/**
	 * 删除现场踏勘记录
	 */
//	@RequiresPermissions("workexplore:workExplore:del")
	@RequestMapping(value = "delete")
	public String delete(WorkExplore workExplore, RedirectAttributes redirectAttributes) {
		workExploreService.delete(workExplore);
		addMessage(redirectAttributes, "删除现场踏勘记录成功");
		return "redirect:"+Global.getAdminPath()+"/workexplore/workExplore/?repage";
	}
	
	/**
	 * 批量删除现场踏勘记录
	 */
//	@RequiresPermissions("workexplore:workExplore:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workExploreService.delete(workExploreService.get(id));
		}
		addMessage(redirectAttributes, "删除现场踏勘记录成功");
		return "redirect:"+Global.getAdminPath()+"/workexplore/workExplore/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workexplore:workExplore:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkExplore workExplore, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "现场踏勘记录"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkExplore> page = workExploreService.findPage(new Page<WorkExplore>(request, response, -1), workExplore);
    		new ExportExcel("现场踏勘记录", WorkExplore.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出现场踏勘记录记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workexplore/workExplore/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workexplore:workExplore:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkExplore> list = ei.getDataList(WorkExplore.class);
			for (WorkExplore workExplore : list){
				try{
					workExploreService.save(workExplore);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条现场踏勘记录记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条现场踏勘记录记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入现场踏勘记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workexplore/workExplore/?repage";
    }
	
	/**
	 * 下载导入现场踏勘记录数据模板
	 */
//	@RequiresPermissions("workexplore:workExplore:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "现场踏勘记录数据导入模板.xlsx";
    		List<WorkExplore> list = Lists.newArrayList(); 
    		new ExportExcel("现场踏勘记录数据", WorkExplore.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workexplore/workExplore/?repage";
    }
	
	
	

}