/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workinfolist.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workinfolist.entity.WorkInfoList;
import com.jeeplus.modules.workinfolist.service.WorkInfoListService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 补充资料清单Controller
 * @author 马鹏波
 * @version 2018-06-07
 */
@Controller
@RequestMapping(value = "${adminPath}/workinfolist/workInfoList")
public class WorkInfoListController extends BaseController {

	@Autowired
	private WorkInfoListService workInfoListService;
	
	@ModelAttribute
	public WorkInfoList get(@RequestParam(required=false) String id) {
		WorkInfoList entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workInfoListService.get(id);
		}
		if (entity == null){
			entity = new WorkInfoList();
		}
		return entity;
	}
	
	/**
	 * 补充资料清单列表页面
	 */
//	@RequiresPermissions("workinfolist:workInfoList:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkInfoList workInfoList, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkInfoList> page = workInfoListService.findPage(new Page<WorkInfoList>(request, response), workInfoList); 
		model.addAttribute("page", page);
		return "modules/workinfolist/workInfoListList";
	}

	/**
	 * 查看，增加，编辑补充资料清单表单页面
	 */
//	@RequiresPermissions(value={"workinfolist:workInfoList:view","workinfolist:workInfoList:add","workinfolist:workInfoList:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkInfoList workInfoList, Model model,HttpServletRequest request) {
		if(!Strings.isNullOrEmpty(workInfoList.getProjectId())){
			workInfoList = workInfoListService.findListByProId(workInfoList);
		}
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/workinfolist/workInfoListFormView";
		}else{
			view = "modules/workinfolist/workInfoListForm";
		}
		model.addAttribute("workInfoList", workInfoList);
		return view;
	}

	/**
	 * 保存补充资料清单
	 */
//	@RequiresPermissions(value={"workinfolist:workInfoList:add","workinfolist:workInfoList:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkInfoList workInfoList, Model model,HttpServletRequest request ,RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workInfoList)){
			return form(workInfoList, model, request);
		}
		if(!workInfoList.getIsNewRecord()){//编辑表单保存
			WorkInfoList t = workInfoListService.get(workInfoList.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workInfoList, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workInfoListService.save(t);//保存
		}else{//新增表单保存
			workInfoListService.save(workInfoList);//保存
		}
		addMessage(redirectAttributes, "保存补充资料清单成功");
		return "redirect:"+Global.getAdminPath()+"/workinfolist/workInfoList/?repage";
	}
	
	/**
	 * 删除补充资料清单
	 */
	@RequiresPermissions("workinfolist:workInfoList:del")
	@RequestMapping(value = "delete")
	public String delete(WorkInfoList workInfoList, RedirectAttributes redirectAttributes) {
		workInfoListService.delete(workInfoList);
		addMessage(redirectAttributes, "删除补充资料清单成功");
		return "redirect:"+Global.getAdminPath()+"/workinfolist/workInfoList/?repage";
	}
	
	/**
	 * 批量删除补充资料清单
	 */
	@RequiresPermissions("workinfolist:workInfoList:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workInfoListService.delete(workInfoListService.get(id));
		}
		addMessage(redirectAttributes, "删除补充资料清单成功");
		return "redirect:"+Global.getAdminPath()+"/workinfolist/workInfoList/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workinfolist:workInfoList:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkInfoList workInfoList, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "补充资料清单"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkInfoList> page = workInfoListService.findPage(new Page<WorkInfoList>(request, response, -1), workInfoList);
    		new ExportExcel("补充资料清单", WorkInfoList.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出补充资料清单记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workinfolist/workInfoList/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workinfolist:workInfoList:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkInfoList> list = ei.getDataList(WorkInfoList.class);
			for (WorkInfoList workInfoList : list){
				try{
					workInfoListService.save(workInfoList);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条补充资料清单记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条补充资料清单记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入补充资料清单失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workinfolist/workInfoList/?repage";
    }
	
	/**
	 * 下载导入补充资料清单数据模板
	 */
//	@RequiresPermissions("workinfolist:workInfoList:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "补充资料清单数据导入模板.xlsx";
    		List<WorkInfoList> list = Lists.newArrayList(); 
    		new ExportExcel("补充资料清单数据", WorkInfoList.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workinfolist/workInfoList/?repage";
    }
	
	
	

}