/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workprojectdeposit.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.sys.utils.UserUtils;
import com.jeeplus.modules.workprojectdeposit.entity.WorkProjectDeposit;
import com.jeeplus.modules.workprojectdeposit.service.WorkProjectDepositService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 保证金模块Controller
 * @author 杨帆
 * @version 2017-12-26
 */
@Controller
@RequestMapping(value = "${adminPath}/workprojectdeposit/workProjectDeposit")
public class WorkProjectDepositController extends BaseController {

	@Autowired
	private WorkProjectDepositService workProjectDepositService;
	
	@ModelAttribute
	public WorkProjectDeposit get(@RequestParam(required=false) String id) {
		WorkProjectDeposit entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workProjectDepositService.get(id);
		}
		if (entity == null){
			entity = new WorkProjectDeposit();
		}
		return entity;
	}
	
	/**
	 * 保证金模块列表页面
	 */
	@RequiresPermissions("workprojectdeposit:workProjectDeposit:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkProjectDeposit workProjectDeposit, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkProjectDeposit> page = workProjectDepositService.findPage(new Page<WorkProjectDeposit>(request, response), workProjectDeposit); 
		model.addAttribute("page", page);
		return "modules/workprojectdeposit/workProjectDepositList";
	}

	/**
	 * 查看，增加，编辑保证金模块表单页面
	 */
	@RequiresPermissions(value={"workprojectdeposit:workProjectDeposit:view","workprojectdeposit:workProjectDeposit:add","workprojectdeposit:workProjectDeposit:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkProjectDeposit workProjectDeposit, Model model,HttpServletRequest request) {
		model.addAttribute("workProjectDeposit", workProjectDeposit);
		model.addAttribute("workBidingProjectId",request.getParameter("workBidingProjectId"));
		return "modules/workprojectdeposit/workProjectDepositForm";
	}

	/**
	 * 保存保证金模块
	 */
	@RequiresPermissions(value={"workprojectdeposit:workProjectDeposit:add","workprojectdeposit:workProjectDeposit:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkProjectDeposit workProjectDeposit, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workProjectDeposit)){
			return form(workProjectDeposit, model,null);
		}
		if(!workProjectDeposit.getIsNewRecord()){//编辑表单保存
			WorkProjectDeposit t = workProjectDepositService.get(workProjectDeposit.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workProjectDeposit, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workProjectDepositService.save(t);//保存
		}else{//新增表单保存
			workProjectDeposit.setDepositStatus("1");
			workProjectDeposit.setBidingDocumentStatus("1");
			workProjectDeposit.setCompanyId(UserUtils.getSelectCompany().getId());
			workProjectDeposit.setOfficeId(UserUtils.getSelectOffice().getId());
			workProjectDepositService.save(workProjectDeposit);//保存
		}
		addMessage(redirectAttributes, "保存保证金模块成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectdeposit/workProjectDeposit/?repage";
	}
	
	/**
	 * 删除保证金模块
	 */
	@RequiresPermissions("workprojectdeposit:workProjectDeposit:del")
	@RequestMapping(value = "delete")
	public String delete(WorkProjectDeposit workProjectDeposit, RedirectAttributes redirectAttributes) {
		workProjectDepositService.delete(workProjectDeposit);
		addMessage(redirectAttributes, "删除保证金模块成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectdeposit/workProjectDeposit/?repage";
	}
	
	/**
	 * 批量删除保证金模块
	 */
	@RequiresPermissions("workprojectdeposit:workProjectDeposit:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workProjectDepositService.delete(workProjectDepositService.get(id));
		}
		addMessage(redirectAttributes, "删除保证金模块成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectdeposit/workProjectDeposit/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workprojectdeposit:workProjectDeposit:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkProjectDeposit workProjectDeposit, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "保证金模块"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkProjectDeposit> page = workProjectDepositService.findPage(new Page<WorkProjectDeposit>(request, response, -1), workProjectDeposit);
    		new ExportExcel("保证金模块", WorkProjectDeposit.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出保证金模块记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectdeposit/workProjectDeposit/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workprojectdeposit:workProjectDeposit:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkProjectDeposit> list = ei.getDataList(WorkProjectDeposit.class);
			for (WorkProjectDeposit workProjectDeposit : list){
				try{
					workProjectDepositService.save(workProjectDeposit);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条保证金模块记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条保证金模块记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入保证金模块失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectdeposit/workProjectDeposit/?repage";
    }
	
	/**
	 * 下载导入保证金模块数据模板
	 */
	@RequiresPermissions("workprojectdeposit:workProjectDeposit:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "保证金模块数据导入模板.xlsx";
    		List<WorkProjectDeposit> list = Lists.newArrayList(); 
    		new ExportExcel("保证金模块数据", WorkProjectDeposit.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectdeposit/workProjectDeposit/?repage";
    }
	
	
	

}