/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.worksalaryitem.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.worksalaryitem.entity.WorkSalaryItem;
import com.jeeplus.modules.worksalaryitem.service.WorkSalaryItemService;

/**
 * 工资项Controller
 * @author ssrh
 * @version 2018-09-17
 */
@Controller
@RequestMapping(value = "${adminPath}/worksalaryitem/workSalaryItem")
public class WorkSalaryItemController extends BaseController {

	@Autowired
	private WorkSalaryItemService workSalaryItemService;
	@Autowired
	private HttpServletRequest request;
	@ModelAttribute
	public WorkSalaryItem get(@RequestParam(required=false) String id) {
		WorkSalaryItem entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workSalaryItemService.get(id);
		}
		if (entity == null){
			entity = new WorkSalaryItem();
		}
		return entity;
	}
	
	/**
	 * 工资项列表页面
	 */
	@RequiresPermissions("worksalaryitem:workSalaryItem:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkSalaryItem workSalaryItem, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkSalaryItem> page = workSalaryItemService.findPage(new Page<WorkSalaryItem>(request, response), workSalaryItem); 
		model.addAttribute("page", page);
		return "modules/worksalaryitem/workSalaryItemList";
	}

	/**
	 * 查看，增加，编辑工资项表单页面
	 */
	@RequiresPermissions(value={"worksalaryitem:workSalaryItem:view","worksalaryitem:workSalaryItem:add","worksalaryitem:workSalaryItem:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkSalaryItem workSalaryItem, Model model) {
		String tabId = request.getParameter("tabId");
		if("1".equals(tabId)){
			model.addAttribute("workSalaryItem", workSalaryItem);
			return "modules/worksalaryitem/workSalaryItemView";
		}
		if(StringUtils.isBlank(workSalaryItem.getId())){
			workSalaryItem.setState("2");
		}
		model.addAttribute("workSalaryItem", workSalaryItem);
		return "modules/worksalaryitem/workSalaryItemForm";
	}

	/**
	 * 保存工资项
	 */
	@RequiresPermissions(value={"worksalaryitem:workSalaryItem:add","worksalaryitem:workSalaryItem:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkSalaryItem workSalaryItem, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workSalaryItem)){
			return form(workSalaryItem, model);
		}
		if(!workSalaryItem.getIsNewRecord()){//编辑表单保存
			WorkSalaryItem t = workSalaryItemService.get(workSalaryItem.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workSalaryItem, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workSalaryItemService.save(t);//保存
		}else{//新增表单保存
			workSalaryItem.setCompanyId(UserUtils.getSelectCompany().getId());
			workSalaryItem.setOfficeId(UserUtils.getSelectOffice().getId());
			//状态 默认启用
			workSalaryItem.setState("2");
			workSalaryItemService.save(workSalaryItem);//保存
		}
		addMessage(redirectAttributes, "保存工资项成功");
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
	}
	
	/**
	 * 删除工资项
	 */
	@RequiresPermissions("worksalaryitem:workSalaryItem:del")
	@RequestMapping(value = "delete")
	public String delete(WorkSalaryItem workSalaryItem, RedirectAttributes redirectAttributes) {
		workSalaryItemService.delete(workSalaryItem);
		addMessage(redirectAttributes, "删除工资项成功");
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
	}
	@RequiresPermissions("worksalaryitem:workSalaryItem:edit")
	@RequestMapping(value = "edits")
	public String edits(WorkSalaryItem workSalaryItem, RedirectAttributes redirectAttributes) {
		String tabId = request.getParameter("tabId");
		if("3".equals(tabId)){
			//禁用
			workSalaryItem.setState("1");
		}else if("4".equals(tabId)){
			//启用
			workSalaryItem.setState("2");
		}
		workSalaryItemService.save(workSalaryItem);
		addMessage(redirectAttributes, "更新成功");
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
	}
	/**
	 * 批量删除工资项
	 */
	@RequiresPermissions("worksalaryitem:workSalaryItem:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workSalaryItemService.delete(workSalaryItemService.get(id));
		}
		addMessage(redirectAttributes, "删除工资项成功");
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("worksalaryitem:workSalaryItem:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkSalaryItem workSalaryItem, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工资项"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkSalaryItem> page = workSalaryItemService.findPage(new Page<WorkSalaryItem>(request, response, -1), workSalaryItem);
    		new ExportExcel("工资项", WorkSalaryItem.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工资项记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("worksalaryitem:workSalaryItem:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkSalaryItem> list = ei.getDataList(WorkSalaryItem.class);
			for (WorkSalaryItem workSalaryItem : list){
				try{
					workSalaryItemService.save(workSalaryItem);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工资项记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工资项记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工资项失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
    }
	
	/**
	 * 下载导入工资项数据模板
	 */
	@RequiresPermissions("worksalaryitem:workSalaryItem:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工资项数据导入模板.xlsx";
    		List<WorkSalaryItem> list = Lists.newArrayList(); 
    		new ExportExcel("工资项数据", WorkSalaryItem.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/worksalaryitem/workSalaryItem/?repage";
    }
	
	
	

}