/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workstaff.service;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jeeplus.common.utils.IdGen;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.sys.dao.UserDao;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.utils.DictUtils;
import com.jeeplus.modules.workstaff.entity.WorkStaffBasicInfo;
import com.jeeplus.modules.workstaff.entity.WorkStaffLanguages;
import com.jeeplus.modules.workstaffachiveslog.entity.WorkStaffAchivesLog;
import com.jeeplus.modules.workstaffachiveslog.service.WorkStaffAchivesLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.modules.workstaff.entity.WorkStaffAchievement;
import com.jeeplus.modules.workstaff.dao.WorkStaffAchievementDao;
import org.springframework.web.multipart.MultipartFile;

/**
 * 工作业绩Service
 * @author ssrh
 * @version 2018-07-27
 */
@Service
@Transactional(readOnly = true)
public class WorkStaffAchievementService extends CrudService<WorkStaffAchievementDao, WorkStaffAchievement> {
    @Autowired
    private WorkStaffAchivesLogService workStaffAchivesLogService;
    @Autowired
    private UserDao userDao;
	public WorkStaffAchievement get(String id) {
		return super.get(id);
	}
	
	public List<WorkStaffAchievement> findList(WorkStaffAchievement workStaffAchievement) {
		return super.findList(workStaffAchievement);
	}
	
	public Page<WorkStaffAchievement> findPage(Page<WorkStaffAchievement> page, WorkStaffAchievement workStaffAchievement) {
		return super.findPage(page, workStaffAchievement);
	}
	
	@Transactional(readOnly = false)
	public void save(WorkStaffAchievement workStaffAchievement) {
		super.save(workStaffAchievement);
	}
	
	@Transactional(readOnly = false)
	public void delete(WorkStaffAchievement workStaffAchievement) {
		super.delete(workStaffAchievement);
	}


    public boolean save(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null || StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        if(workStaffBasicInfo.getAchievementList()==null||workStaffBasicInfo.getAchievementList().size()<=0){
            return false;
        }
        boolean flag = false;
        for (WorkStaffAchievement entity : workStaffBasicInfo.getAchievementList()) {
            if(entity!=null&&"1".equals(entity.getDelFlag())&&StringUtils.isBlank(entity.getId())){
                continue;
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            if(entity!=null){
                entity.setStaff(workStaffBasicInfo.getId());
                this.save(entity);
                if(first){
                    entity.setId(null);
                }
                flag =true;
            }
        }
        return flag;
    }

    public void query(WorkStaffBasicInfo workStaffBasicInfo) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffAchievement entity = new WorkStaffAchievement();
        entity.setStaff(workStaffBasicInfo.getId());
        workStaffBasicInfo.setAchievementList(this.findList(entity));
    }
    private static Map<String,String> EDU_MAP = Maps.newHashMap();//教育经历

    static{//教育经历
        EDU_MAP.put("projName", "项目名称");
        EDU_MAP.put("projClient", "主委托方");
        EDU_MAP.put("category", "工程分类");
        EDU_MAP.put("unitNumber", "规模类型/单位/数量");
        EDU_MAP.put("duty", "担任职务");
        EDU_MAP.put("content", "参与内容");
    }

    public boolean saveApply(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffAchievement> list = workStaffBasicInfo.getAchievementList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag = false;
        User u=userDao.get(workStaffBasicInfo.getUserId());
        for (WorkStaffAchievement entity : list) {
            entity.setStaff(workStaffBasicInfo.getId());
            WorkStaffAchievement workStaffLanguages=dao.get(entity.getId());
            if(workStaffLanguages==null){
                entity.setCreateBy(u);
                entity.setCreateDate(new Date());
                entity.setUpdateBy(u);
                entity.setUpdateDate(new Date());
                dao.insert(entity);
            }else{
                this.save(entity);
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            flag =true;
        }
        dao.updateFalt(workStaffBasicInfo.getId());
        return flag;
    }
    public void querys(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffAchievement entity = new WorkStaffAchievement();
        entity.setStaff(workStaffBasicInfo.getId());
        List<WorkStaffAchievement> list=this.findList(entity);
        List<WorkStaffAchievement> newList=Lists.newArrayList();
        if(list!=null&&list.size()>0){//修改
            for(WorkStaffAchievement workStaffAchievement:list){
                Class<? extends WorkStaffAchievement> newClass = workStaffAchievement.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("projName".equals(name)||"projClient".equals(name)||"category".equals(name)||"unitNumber".equals(name)||"duty".equals(name)
                            || "content".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                        workStaffAchivesLog.setModule("工作业绩");
                        workStaffAchivesLog.setSonId(workStaffAchievement.getId());
                        workStaffAchivesLog.setFields(name);
                        workStaffAchivesLog.setType("修改");
                        workStaffAchivesLog.setState("1");
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                            Object newInvoke = newClass.getMethod(getMethod).invoke(workStaffAchievement);//修改后
                            if(newInvoke instanceof String){
                                Method setReadOnly = workStaffAchievement.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffAchievement,logs.get(0).getNewKey());
                                }
                            }
                            if(newInvoke instanceof Date){//日期
                                Method setReadOnly = workStaffAchievement.getClass().getMethod(setMethod, Date.class);
                                if(logs!=null&&logs.size()>0){
                                    Date myDate = dateFormat1.parse(logs.get(0).getNewKey());
                                    setReadOnly.invoke(workStaffAchievement,myDate);
                                }
                            }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffAchievement);
            }
        }
        //新增
        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
        workStaffAchivesLog.setModule("工作业绩");
        workStaffAchivesLog.setType("新增");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> sonIds=workStaffAchivesLogService.findSonId(workStaffAchivesLog);
        if(sonIds!=null&&sonIds.size()>0){
            for(WorkStaffAchivesLog w:sonIds){
                workStaffAchivesLog.setSonId(w.getSonId());
                WorkStaffAchievement workStaffAchievement = new WorkStaffAchievement();
                workStaffAchievement.setId(w.getSonId());
                workStaffAchievement.setDelFlag("0");
                workStaffAchievement.setStaff(workStaffBasicInfo.getId());
                Class<? extends WorkStaffAchievement> newClass = workStaffAchievement.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("projName".equals(name)||"projClient".equals(name)||"category".equals(name)||"unitNumber".equals(name)||"duty".equals(name)
                            || "content".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        workStaffAchivesLog.setFields(name);
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                                Method setReadOnly = workStaffAchievement.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffAchievement,logs.get(0).getNewKey());
                                }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffAchievement);
            }
        }
        //删除
        WorkStaffAchivesLog delLog=new WorkStaffAchivesLog();
        delLog.setStaffId(workStaffBasicInfo.getId());
        delLog.setModule("工作业绩");
        delLog.setType("删除");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> dellogs=workStaffAchivesLogService.findList(delLog);
        if(dellogs!=null&&dellogs.size()>0){
            for(WorkStaffAchivesLog w:dellogs){
                newList.remove(dao.get(w.getSonId()));
            }
        }
        workStaffBasicInfo.setAchievementList(newList);
    }
    @Transactional(readOnly = false)
    public void saveEdu(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<WorkStaffAchievement> achievementList =workStaffBasicInfo.getAchievementList();
        if(achievementList!=null&&achievementList.size()>0){
            for(WorkStaffAchievement newAchievement:achievementList){
                Class<? extends WorkStaffAchievement> newClass = newAchievement.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                if(newAchievement.getId()!=null&&!"1".equals(newAchievement.getDelFlag())&&!"".equals(newAchievement.getId())){
                    WorkStaffAchievement oldAchievement=this.get(newAchievement.getId());
                    if(oldAchievement!=null){
                        Class<? extends WorkStaffAchievement> oldClass = oldAchievement.getClass();
                        for (Field field : declaredFields) {
                            String name = field.getName();
                            if("projName".equals(name)||"projClient".equals(name)||"category".equals(name)||"unitNumber".equals(name)||"duty".equals(name)
                                    || "content".equals(name)){
                                String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                                WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                                workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                                workStaffAchivesLog.setModule("工作业绩");
                                workStaffAchivesLog.setSonId(newAchievement.getId());
                                workStaffAchivesLog.setFields(name);
                                workStaffAchivesLog.setState("1");
                                try {
                                    Object newInvoke = newClass.getMethod(methodName).invoke(newAchievement);//修改后
                                    Object oldInvoke = oldClass.getMethod(methodName).invoke(oldAchievement);//修改前
                                    String describes=EDU_MAP.get(name);
                                    if(newInvoke instanceof String){
                                        String newValue=newInvoke.toString();
                                        String oldValue=oldInvoke.toString();
                                        if(!newValue.equals(oldValue)){
                                                workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                        "工作业绩",describes,name,
                                                        oldInvoke.toString(),newInvoke.toString(),
                                                        oldInvoke.toString(),newInvoke.toString(),newAchievement.getId(),"修改");

                                        }
                                    }
                                    if(newInvoke instanceof Date){//日期
                                        String newDate="";
                                        String oldDate="";
                                        if(newInvoke!=null){
                                            newDate=sdf.format(newInvoke);
                                        }
                                        if (oldInvoke!=null){
                                            oldDate=sdf.format(oldInvoke);
                                        }
                                        if(!newDate.equals(oldDate)){
                                            workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                    "工作业绩",describes,name,
                                                    oldDate,newDate,oldDate,newDate,newAchievement.getId(),"修改");
                                        }
                                    }
                                } catch (NoSuchMethodException e) {
                                    e.printStackTrace();
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                } catch (InvocationTargetException e) {
                                    e.printStackTrace();
                                }
                            }
                        }

                    }
                }else if((newAchievement.getId()==null||"".equals(newAchievement.getId()))&&"0".equals(newAchievement.getDelFlag())){//新增
                    String newId=IdGen.uuid();
                    for (Field field : declaredFields) {
                        String name = field.getName();
                        if("projName".equals(name)||"projClient".equals(name)||"category".equals(name)||"unitNumber".equals(name)||"duty".equals(name)
                                || "content".equals(name)){
                            String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                            WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                            workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                            workStaffAchivesLog.setModule("工作业绩");
                            workStaffAchivesLog.setSonId(newId);
                            workStaffAchivesLog.setFields(name);
                            workStaffAchivesLog.setState("1");
                            try {
                                Object newInvoke = newClass.getMethod(methodName).invoke(newAchievement);//修改后
                                String describes=EDU_MAP.get(name);
                                if(newInvoke instanceof String){
                                    String newValue=newInvoke.toString();
                                        workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                "工作业绩",describes,name,
                                                "",newInvoke.toString(),
                                                "",newInvoke.toString(),newId,"新增");

                                }
                                if(newInvoke instanceof Date){//日期
                                    String newDate=sdf.format(newInvoke);
                                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                            "工作业绩",describes,name,
                                            "",newDate,"",newDate,newId,"新增");
                                };
                            } catch (NoSuchMethodException e) {
                                e.printStackTrace();
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                            } catch (InvocationTargetException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }else if(newAchievement.getId()!=null&&"1".equals(newAchievement.getDelFlag())&&!"".equals(newAchievement.getId())){//删除
                    WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                    workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                    workStaffAchivesLog.setModule("工作业绩");
                    workStaffAchivesLog.setSonId(newAchievement.getId());
                    workStaffAchivesLog.setType("删除");
                    workStaffAchivesLog.setState("1");
                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                            "工作业绩","","",
                            "","",
                            "","",newAchievement.getId(),"删除");
                }
            }
        }
    }
}