/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workstaff.service;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jeeplus.common.utils.IdGen;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.sys.dao.UserDao;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.utils.DictUtils;
import com.jeeplus.modules.workstaff.entity.WorkStaffBasicInfo;
import com.jeeplus.modules.workstaff.entity.WorkStaffEducation;
import com.jeeplus.modules.workstaffachiveslog.entity.WorkStaffAchivesLog;
import com.jeeplus.modules.workstaffachiveslog.service.WorkStaffAchivesLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.modules.workstaff.entity.WorkStaffSocialPosition;
import com.jeeplus.modules.workstaff.dao.WorkStaffSocialPositionDao;
import org.springframework.web.multipart.MultipartFile;

/**
 * 社会及行业职务Service
 * @author ssrh
 * @version 2018-07-27
 */
@Service
@Transactional(readOnly = true)
public class WorkStaffSocialPositionService extends CrudService<WorkStaffSocialPositionDao, WorkStaffSocialPosition> {
    @Autowired
    private WorkStaffAchivesLogService workStaffAchivesLogService;
    @Autowired
    private UserDao userDao;
	public WorkStaffSocialPosition get(String id) {
		return super.get(id);
	}
	
	public List<WorkStaffSocialPosition> findList(WorkStaffSocialPosition workStaffSocialPosition) {
		return super.findList(workStaffSocialPosition);
	}
	
	public Page<WorkStaffSocialPosition> findPage(Page<WorkStaffSocialPosition> page, WorkStaffSocialPosition workStaffSocialPosition) {
		return super.findPage(page, workStaffSocialPosition);
	}
	
	@Transactional(readOnly = false)
	public void save(WorkStaffSocialPosition workStaffSocialPosition) {
		super.save(workStaffSocialPosition);
	}
	
	@Transactional(readOnly = false)
	public void delete(WorkStaffSocialPosition workStaffSocialPosition) {
		super.delete(workStaffSocialPosition);
	}


    public boolean save(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffSocialPosition> list = workStaffBasicInfo.getSocialPositionList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag=false;
        for (WorkStaffSocialPosition entity : list) {
            if(entity!=null&&"1".equals(entity.getDelFlag())&&StringUtils.isBlank(entity.getId())){
                continue;
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            if (entity!=null){
                entity.setStaff(workStaffBasicInfo.getId());
                this.save(entity);
                if(first){
                    entity.setId(null);
                }
                flag = true;
            }
        }
        return flag;
    }

    public void query(WorkStaffBasicInfo workStaffBasicInfo) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffSocialPosition entity = new WorkStaffSocialPosition();
        entity.setStaff(workStaffBasicInfo.getId());
        workStaffBasicInfo.setSocialPositionList(this.findList(entity));
    }
    private static Map<String,String> EDU_MAP = Maps.newHashMap();//教育经历

    static{//教育经历
        EDU_MAP.put("name", "职务名称");
        EDU_MAP.put("startDate", "任职开始日期");
        EDU_MAP.put("endDate", "任职结束日期");
        EDU_MAP.put("organization", "任职机构");
    }
    public boolean saveApply(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffSocialPosition> list = workStaffBasicInfo.getSocialPositionList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag = false;
        User u=userDao.get(workStaffBasicInfo.getUserId());
        for (WorkStaffSocialPosition entity : list) {
            entity.setStaff(workStaffBasicInfo.getId());
            WorkStaffSocialPosition workStaffSocialPosition=dao.get(entity.getId());
            if(workStaffSocialPosition==null){
                entity.setCreateBy(u);
                entity.setCreateDate(new Date());
                entity.setUpdateBy(u);
                entity.setUpdateDate(new Date());
                dao.insert(entity);
            }else{
                this.save(entity);
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            flag =true;
        }
        dao.updateFalt(workStaffBasicInfo.getId());
        return flag;
    }
    public void querys(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffSocialPosition entity = new WorkStaffSocialPosition();
        entity.setStaff(workStaffBasicInfo.getId());
        List<WorkStaffSocialPosition> list=this.findList(entity);
        List<WorkStaffSocialPosition> newList=Lists.newArrayList();
        if(list!=null&&list.size()>0){//修改
            for(WorkStaffSocialPosition workStaffSocialPosition:list){
                Class<? extends WorkStaffSocialPosition> newClass = workStaffSocialPosition.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("name".equals(name)||"startDate".equals(name)||"endDate".equals(name)||"organization".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                        workStaffAchivesLog.setModule("社会及行业职务");
                        workStaffAchivesLog.setSonId(workStaffSocialPosition.getId());
                        workStaffAchivesLog.setFields(name);
                        workStaffAchivesLog.setType("修改");
                        workStaffAchivesLog.setState("1");
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                            Object newInvoke = newClass.getMethod(getMethod).invoke(workStaffSocialPosition);//修改后
                            if(newInvoke instanceof String){
                                Method setReadOnly = workStaffSocialPosition.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffSocialPosition,logs.get(0).getNewKey());
                                }
                            }
                            if(newInvoke instanceof Date){//日期
                                Method setReadOnly = workStaffSocialPosition.getClass().getMethod(setMethod, Date.class);
                                if(logs!=null&&logs.size()>0){
                                    Date myDate = dateFormat1.parse(logs.get(0).getNewKey());
                                    setReadOnly.invoke(workStaffSocialPosition,myDate);
                                }
                            }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffSocialPosition);
            }
        }
        //新增
        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
        workStaffAchivesLog.setModule("社会及行业职务");
        workStaffAchivesLog.setType("新增");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> sonIds=workStaffAchivesLogService.findSonId(workStaffAchivesLog);
        if(sonIds!=null&&sonIds.size()>0){
            for(WorkStaffAchivesLog w:sonIds){
                workStaffAchivesLog.setSonId(w.getSonId());
                WorkStaffSocialPosition workStaffSocialPosition = new WorkStaffSocialPosition();
                workStaffSocialPosition.setId(w.getSonId());
                workStaffSocialPosition.setDelFlag("0");
                workStaffSocialPosition.setStaff(workStaffBasicInfo.getId());
                Class<? extends WorkStaffSocialPosition> newClass = workStaffSocialPosition.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("name".equals(name)||"startDate".equals(name)||"endDate".equals(name)||"organization".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        workStaffAchivesLog.setFields(name);
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                            if("startDate".equals(name)||"endDate".equals(name)){//日期
                                Method setReadOnly = workStaffSocialPosition.getClass().getMethod(setMethod, Date.class);
                                if(logs!=null&&logs.size()>0){
                                    Date myDate = dateFormat1.parse(logs.get(0).getNewKey());
                                    setReadOnly.invoke(workStaffSocialPosition,myDate);
                                }
                            }else{
                                Method setReadOnly = workStaffSocialPosition.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffSocialPosition,logs.get(0).getNewKey());
                                }
                            }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffSocialPosition);
            }
        }
        //删除
        WorkStaffAchivesLog delLog=new WorkStaffAchivesLog();
        delLog.setStaffId(workStaffBasicInfo.getId());
        delLog.setModule("社会及行业职务");
        delLog.setType("删除");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> dellogs=workStaffAchivesLogService.findList(delLog);
        if(dellogs!=null&&dellogs.size()>0){
            for(WorkStaffAchivesLog w:dellogs){
                newList.remove(dao.get(w.getSonId()));
            }
        }
        workStaffBasicInfo.setSocialPositionList(newList);
    }
    @Transactional(readOnly = false)
    public void saveEdu(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<WorkStaffSocialPosition> socialPositionList =workStaffBasicInfo.getSocialPositionList();
        if(socialPositionList!=null&&socialPositionList.size()>0){
            for(WorkStaffSocialPosition newSocialPosition:socialPositionList){
                Class<? extends WorkStaffSocialPosition> newClass = newSocialPosition.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                if(newSocialPosition.getId()!=null&&!"1".equals(newSocialPosition.getDelFlag())&&!"".equals(newSocialPosition.getId())){
                    WorkStaffSocialPosition oldSocialPosition=this.get(newSocialPosition.getId());
                    if(oldSocialPosition!=null){
                        Class<? extends WorkStaffSocialPosition> oldClass = oldSocialPosition.getClass();
                        for (Field field : declaredFields) {
                            String name = field.getName();
                            if("name".equals(name)||"startDate".equals(name)||"endDate".equals(name)||"organization".equals(name)){
                                String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                                WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                                workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                                workStaffAchivesLog.setModule("社会及行业职务");
                                workStaffAchivesLog.setSonId(newSocialPosition.getId());
                                workStaffAchivesLog.setFields(name);
                                workStaffAchivesLog.setState("1");
                                try {
                                    Object newInvoke = newClass.getMethod(methodName).invoke(newSocialPosition);//修改后
                                    Object oldInvoke = oldClass.getMethod(methodName).invoke(oldSocialPosition);//修改前
                                    String describes=EDU_MAP.get(name);
                                    if(newInvoke instanceof String){
                                        String newValue=newInvoke.toString();
                                        String oldValue=oldInvoke.toString();
                                        if(!newValue.equals(oldValue)){
                                                workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                        "社会及行业职务",describes,name,
                                                        oldInvoke.toString(),newInvoke.toString(),
                                                        oldInvoke.toString(),newInvoke.toString(),newSocialPosition.getId(),"修改");
                                        }
                                    }
                                    if(newInvoke instanceof Date){//日期
                                        String newDate="";
                                        String oldDate="";
                                        if(newInvoke!=null){
                                            newDate=sdf.format(newInvoke);
                                        }
                                        if (oldInvoke!=null){
                                            oldDate=sdf.format(oldInvoke);
                                        }
                                        if(!newDate.equals(oldDate)){
                                            workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                    "社会及行业职务",describes,name,
                                                    oldDate,newDate,oldDate,newDate,newSocialPosition.getId(),"修改");
                                        }
                                    }
                                } catch (NoSuchMethodException e) {
                                    e.printStackTrace();
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                } catch (InvocationTargetException e) {
                                    e.printStackTrace();
                                }
                            }
                        }
                    }
                }else if((newSocialPosition.getId()==null||"".equals(newSocialPosition.getId()))&&"0".equals(newSocialPosition.getDelFlag())){//新增
                    String newId=IdGen.uuid();
                    for (Field field : declaredFields) {
                        String name = field.getName();
                        if("name".equals(name)||"startDate".equals(name)||"endDate".equals(name)||"organization".equals(name)){
                            String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                            WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                            workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                            workStaffAchivesLog.setModule("社会及行业职务");
                            workStaffAchivesLog.setSonId(newId);
                            workStaffAchivesLog.setFields(name);
                            workStaffAchivesLog.setState("1");
                            try {
                                Object newInvoke = newClass.getMethod(methodName).invoke(newSocialPosition);//修改后
                                String describes=EDU_MAP.get(name);
                                if(newInvoke instanceof String){
                                    String newValue=newInvoke.toString();
                                        workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                "社会及行业职务",describes,name,
                                                "",newInvoke.toString(),
                                                "",newInvoke.toString(),newId,"新增");
                                }
                                if(newInvoke instanceof Date){//日期
                                    String newDate=sdf.format(newInvoke);
                                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                            "社会及行业职务",describes,name,
                                            "",newDate,"",newDate,newId,"新增");
                                };
                            } catch (NoSuchMethodException e) {
                                e.printStackTrace();
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                            } catch (InvocationTargetException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }else if(newSocialPosition.getId()!=null&&"1".equals(newSocialPosition.getDelFlag())&&!"".equals(newSocialPosition.getId())){//删除
                    WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                    workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                    workStaffAchivesLog.setModule("社会及行业职务");
                    workStaffAchivesLog.setSonId(newSocialPosition.getId());
                    workStaffAchivesLog.setType("删除");
                    workStaffAchivesLog.setState("1");
                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                            "社会及行业职务","","",
                            "","",
                            "","",newSocialPosition.getId(),"删除");
                }
            }
        }
    }
}