/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workstaffachiveslog.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workstaffachiveslog.entity.WorkStaffAchivesLog;
import com.jeeplus.modules.workstaffachiveslog.service.WorkStaffAchivesLogService;

/**
 * 档案修改日志Controller
 * @author ssrh
 * @version 2018-10-11
 */
@Controller
@RequestMapping(value = "${adminPath}/workstaffachiveslog/workStaffAchivesLog")
public class WorkStaffAchivesLogController extends BaseController {

	@Autowired
	private WorkStaffAchivesLogService workStaffAchivesLogService;
	
	@ModelAttribute
	public WorkStaffAchivesLog get(@RequestParam(required=false) String id) {
		WorkStaffAchivesLog entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workStaffAchivesLogService.get(id);
		}
		if (entity == null){
			entity = new WorkStaffAchivesLog();
		}
		return entity;
	}
	
	/**
	 * 档案修改日志列表页面
	 */
	@RequiresPermissions("workstaffachiveslog:workStaffAchivesLog:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkStaffAchivesLog workStaffAchivesLog, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkStaffAchivesLog> page = workStaffAchivesLogService.findPage(new Page<WorkStaffAchivesLog>(request, response), workStaffAchivesLog); 
		model.addAttribute("page", page);
		return "modules/workstaffachiveslog/workStaffAchivesLogList";
	}

	/**
	 * 查看，增加，编辑档案修改日志表单页面
	 */
	@RequiresPermissions(value={"workstaffachiveslog:workStaffAchivesLog:view","workstaffachiveslog:workStaffAchivesLog:add","workstaffachiveslog:workStaffAchivesLog:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkStaffAchivesLog workStaffAchivesLog, Model model) {
		model.addAttribute("workStaffAchivesLog", workStaffAchivesLog);
		return "modules/workstaffachiveslog/workStaffAchivesLogForm";
	}

	/**
	 * 保存档案修改日志
	 */
	@RequiresPermissions(value={"workstaffachiveslog:workStaffAchivesLog:add","workstaffachiveslog:workStaffAchivesLog:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkStaffAchivesLog workStaffAchivesLog, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workStaffAchivesLog)){
			return form(workStaffAchivesLog, model);
		}
		if(!workStaffAchivesLog.getIsNewRecord()){//编辑表单保存
			WorkStaffAchivesLog t = workStaffAchivesLogService.get(workStaffAchivesLog.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workStaffAchivesLog, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workStaffAchivesLogService.save(t);//保存
		}else{//新增表单保存
			workStaffAchivesLogService.save(workStaffAchivesLog);//保存
		}
		addMessage(redirectAttributes, "保存档案修改日志成功");
		return "redirect:"+Global.getAdminPath()+"/workstaffachiveslog/workStaffAchivesLog/?repage";
	}
	
	/**
	 * 删除档案修改日志
	 */
	@RequiresPermissions("workstaffachiveslog:workStaffAchivesLog:del")
	@RequestMapping(value = "delete")
	public String delete(WorkStaffAchivesLog workStaffAchivesLog, RedirectAttributes redirectAttributes) {
		workStaffAchivesLogService.delete(workStaffAchivesLog);
		addMessage(redirectAttributes, "删除档案修改日志成功");
		return "redirect:"+Global.getAdminPath()+"/workstaffachiveslog/workStaffAchivesLog/?repage";
	}
	
	/**
	 * 批量删除档案修改日志
	 */
	@RequiresPermissions("workstaffachiveslog:workStaffAchivesLog:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workStaffAchivesLogService.delete(workStaffAchivesLogService.get(id));
		}
		addMessage(redirectAttributes, "删除档案修改日志成功");
		return "redirect:"+Global.getAdminPath()+"/workstaffachiveslog/workStaffAchivesLog/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workstaffachiveslog:workStaffAchivesLog:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkStaffAchivesLog workStaffAchivesLog, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "档案修改日志"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkStaffAchivesLog> page = workStaffAchivesLogService.findPage(new Page<WorkStaffAchivesLog>(request, response, -1), workStaffAchivesLog);
    		new ExportExcel("档案修改日志", WorkStaffAchivesLog.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出档案修改日志记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workstaffachiveslog/workStaffAchivesLog/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workstaffachiveslog:workStaffAchivesLog:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkStaffAchivesLog> list = ei.getDataList(WorkStaffAchivesLog.class);
			for (WorkStaffAchivesLog workStaffAchivesLog : list){
				try{
					workStaffAchivesLogService.save(workStaffAchivesLog);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条档案修改日志记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条档案修改日志记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入档案修改日志失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workstaffachiveslog/workStaffAchivesLog/?repage";
    }
	
	/**
	 * 下载导入档案修改日志数据模板
	 */
	@RequiresPermissions("workstaffachiveslog:workStaffAchivesLog:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "档案修改日志数据导入模板.xlsx";
    		List<WorkStaffAchivesLog> list = Lists.newArrayList(); 
    		new ExportExcel("档案修改日志数据", WorkStaffAchivesLog.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workstaffachiveslog/workStaffAchivesLog/?repage";
    }
	
	
	

}